function gui_skeleton(varargin)
% GUI_SKELETON.M
% One approach for writing a simple application
% Usage:
% gui_skeleton
%   with no arguments, it will read a default image and let you interact
%   with it without blocking the command line
% gui_skeleton(parameter,value{,parameter,value})
%    you can specify some arguments to override the defaults if you prefer
%
% Parameter choices:
%  filename  - specify an image to read (anything imread can use)
%  clims     - specify a vector [cmin cmax] of colour limits
%
% Interactions:
% (You can see these in the sfKeyPress routine)
% cursor keys - modify brightness and contrast
% r           - reset colourmap to automatic
% c           - turn on GUI for colourmap adjustment
% m           - makes a little movie and writes it to 'movie.avi'
% <enter>     - writes the image to the variable 'im' in the base workspace
% q           - quits cleanly
%
% Mouse interactions:
% click and drag the mouse to see the mouse callbacks in action (positions
% are in pixels


app=[]; % initialise app so it is visible in all nested functions
sfInitialiseFigure(varargin);
sfUpdateFigure;
%help gui_skeleton

% sfNestedOrNot;  % 
% end of main function

% beginning of nested subfunctions

%% sfInitialiseFigure
  function sfInitialiseFigure(arguments)
    warning('off','Images:initSize:adjustingMag');
    warning('off','MATLAB:divideByZero')

    % set defaults
    app.filename='/sware/com/matlab_r14sp3/toolbox/images/imdemos/rice.png';
    app.clims=[];
     
    % if the user overrode a default, use that
    app=parse_pv_pairs(app,arguments);

    app.handles.fig=figure;
    set(app.handles.fig,'numbertitle','off','menubar','none');
    %set callbacks for keypresses
    set(app.handles.fig,'keypressfcn',@sfKeyPress);
    set(app.handles.fig,'CloseRequestFcn',@sfQuit);
    % set callbacks for mouse (more than you need - just for example)

    % this one is active ANYWHERE in the figure window
    set(app.handles.fig,'WindowButtonDownFcn',@sfMouseClick);
    set(app.handles.fig,'WindowButtonUpFcn',@sfMouseRelease);

    app.firsttime=true;

  end
 function sfMouseClick(varargin)
    src=varargin{1}
    fprintf('You clicked the mouse in figure %d\n',src);
    set(app.handles.fig,'WindowButtonMotionFcn',@sfMouseMoving);
 end
  function sfMouseMoving(varargin)
    pt=get(gca,'currentpoint');
    pt=pt([1 3]);  % the rest of the structure is for Z and 3D stuff
    fprintf('I''m moving! X:%3.1f Y:%3.1f\n',pt(1),pt(2)) % in pixels
  end
  function sfMouseRelease(varargin)
    % stop the mouse motion callback (otherwise it will trigger ALL the
    % time
    set(app.handles.fig,'WindowButtonMotionFcn',[]);
    fprintf('\nYou released the mouse\n')
  end
 
%%  sfUpdateFigure
  function sfUpdateFigure(varargin)
    if app.firsttime
      % read the image and put onscreen for the first time
      app.im=imread(app.filename);
      app.clims=stretchlim2(app.im); 

      figure(app.handles.fig)
      set(app.handles.fig,'name',app.filename)
      % create axes and image (could use imshow instead)
      app.handles.im=imagesc(app.im,app.clims);
      axis image,axis off
      app.handles.ax=gca;
      % and other mouse callbacks in the axes, not just the figure
      set(app.handles.ax,'ButtonDownFcn','3*5');

      colormap(gray)
      % don't need to do this again!
      app.firsttime=false;
    else % any other time we only need to change the colour limits
      set(app.handles.ax,'clim',app.clims);
    end
  end

%%  sfKeyPress
  function sfKeyPress(varargin)
    % varargin{2} contains the event details
    c=varargin{2};

    switch c.Key
      case 'uparrow'
        disp('Increasing brightness')
        app.clims=app.clims-max(app.clims)*0.05;
        sfUpdateFigure
      case 'downarrow'
        disp('Decreasing brightness')
        app.clims=app.clims+max(app.clims)*0.05;
        sfUpdateFigure;

      case 'rightarrow'
        disp('Increasing contrast')
        app.clims=(app.clims-mean(app.clims))*0.95 + mean(app.clims);
        sfUpdateFigure;
      
      case 'leftarrow'
        disp('Decreasing contrast')
        app.clims=(app.clims-mean(app.clims))*1.05 + mean(app.clims);
        sfUpdateFigure;

      case 'm'
        % make a movie, and store it to disk
        disp('Creating movie')
        sfMakeMovie

      case 'return'
        % write image to workspace 'im' variable
        disp('Variable ''im'' now contains image');
        assignin('base','im',app.im);

        % COLOURMAP CHANGES
      case 'r'
        disp('Resetting colormap')
        app.clims=stretchlim2(app.im,0.02);
        sfUpdateFigure
      case 'c'
        % attach the imcontrast tool
        app.handles.contrast=imcontrast;
       
        iptwindowalign(app.handles.fig,'left',app.handles.contrast,'right');
        iptwindowalign(app.handles.fig,'vcenter',app.handles.contrast,'vcenter');

        
        figure(app.handles.fig);
        imdisplayrange

      case 'q' % QUIT
        sfQuit
    end
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfQuit(varargin)
    disp('Quitting')
    delete(gcf);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfMakeMovie
    disp('Opening movie file')
    moviefname='movie.avi';
    mov=avifile(moviefname);
    
    for n=1:100
      app.clims=app.clims./2;
      sfUpdateFigure
      mov=addframe(mov,getframe(gca));
    end
    disp('Saving...')
    mov=close(mov);
    disp(['Movie saved to ' moviefname])
    app.clims=stretchlim2(app.im);
    sfUpdateFigure
  end % end of sfMakeMovie


function sfNestedOrNot
if exist('app','var')
  disp('I can see the ''app'' variable')
else
  disp('I cannot see the ''app'' variable!')
end
end % end of sfNestedOrNot

end % end of gui_skeleton and nested subfunctions
